function idx = stratisample(X, Nsamples, w)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    Called by label_samples.m
%    Performs stratified sampling of dataset, to reduce the variance of the output.
%    Define strata by sequentially splitting dataset along widest dimension.
%    Main benefit is to ensure roughly equal sampling of dataset over time
%    inputs:
%        X - NXD matrix, values to be sampled using stratified sampler
%        Nsamples - int, how many samples to draw
%        w - NX1 vector, sampling weight of each sample
%    outputs:
%        idx - NsamplesX1 vector, random sample indicies
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


	%if no weight, then set equal weight
	if not(exist('w'))
		w = ones(size(X,1),1);
	end

	%if onyl one sample then just draw it
	if Nsamples==1,

		idx = randsample(size(X,1),1,true,w);	

	%if zero samples then return an empty vector
	elseif Nsamples==0,

		idx = [];

	else

		%if all w <=0 then sample nothing and return an empty vector
		if not(any(w>0))
			idx = [];
		else

			%pick random dimension, based on dimension width
                        if size(X,2)>1,
				XSD = sqrt(var(X(logical(w),:)))+realmin;
				dimension = randsample(length(XSD),1,true,XSD);
 			else
				dimension = 1;
			end	

			%split along median and set weights to reflect this
			split = median(X(logical(w),dimension));
			w1 = w.*(X(:,dimension)>split);
			w2 = w.*(X(:,dimension)<split);
	
			%randomly assign those equal to the median. This cna happen as X may have int values
			randassign = rand(size(w))<0.5;
			w1 = w1 + w.*randassign.*(X(:,dimension)==split);
			w2 = w2 + w.*not(randassign).*(X(:,dimension)==split);
	
			%get relative proportions to sample from each set, based on set size. this will be approx 50:50 while Nsamples small wrt sum(w), but needs more care if Nsamples approaches sum(w)
			maxN1 = sum(w1);
			maxN2 = sum(w2);
			N1 = min(round(Nsamples*maxN1/(maxN1+maxN2)), maxN1);
			N2 = min(Nsamples - N1, maxN2);
			N1 = Nsamples - N2;

			%recursive call to sample each split. This carries on until the size of the split is <=1
			idx1 = stratisample(X, N1, w1);
			idx2 = stratisample(X, N2, w2);

			%concatenate the outputs
			idx = [idx1; idx2];

                end

	end




end

